package com.example.application.views.results;

import com.example.application.data.bean.CalculatedMatch;
import com.example.application.data.service.MatchService;
import com.example.application.data.service.MatchdayService;
import com.example.application.data.service.SeasonService;
import com.example.application.utils.StringUtils;
import com.example.application.utils.VaadinUtils;
import com.example.application.views.abstractnavigation.SeasonAndMatchdayNavigationView;
import com.example.application.views.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.ComponentEventListener;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;
import org.springframework.beans.factory.annotation.Autowired;

@CssImport("./views/results/results-view.css")
@Route(value = "results", layout = MainView.class)
@PageTitle("Schachliga DACH - Results")
public class ResultsView extends SeasonAndMatchdayNavigationView {

    private final MatchService matchService;

    private VerticalLayout matchdayLayout;
    private Label matchdayHeader;
    private Grid<CalculatedMatch> grid;

    private Button prevButton;
    private Button nextButton;

    public ResultsView(@Autowired SeasonService seasonService, @Autowired MatchdayService matchdayService, @Autowired MatchService matchService) {
        super(seasonService, matchdayService);
        this.matchService = matchService;
        addClassName("results-view");
    }

    @Override
    protected String route() {
        return "results";
    }

    @Override
    protected boolean showOnlyMatchdaysWithActivity() {
        return false;
    }

    ////////////
    // LAYOUT //
    ////////////

    private VerticalLayout getMatchdayLayout() {
        if (matchdayLayout == null) {
            matchdayLayout = new VerticalLayout();
        }
        return matchdayLayout;
    }

    private Label getMatchdayHeader() {
        if (matchdayHeader == null) {
            matchdayHeader = new Label();
        }
        return matchdayHeader;
    }

    private Grid<CalculatedMatch> getGrid() {
        if (grid == null) {
            grid = new Grid<>();
        }
        return grid;
    }

    private Button getPrevButton() {
        if (prevButton == null) {
            prevButton = new Button(new Icon(VaadinIcon.ARROW_LEFT));
        }
        return prevButton;
    }

    private Button getNextButton() {
        if (nextButton == null) {
            nextButton = new Button(new Icon(VaadinIcon.ARROW_RIGHT));
        }
        return nextButton;
    }

    @Override
    protected void configureContentLayout() {
        contentLayout = getMatchdayLayout();
        configureMatchdayLayout();
    }

    private void configureMatchdayLayout() {
        getMatchdayHeader().addClassName("matchday_header"); // TODO: add dates
        getMatchdayHeader().setWidth("100%");
        HorizontalLayout matchdayHeaderLayout = new HorizontalLayout(getPrevButton(), getMatchdayHeader(), getNextButton());
        matchdayHeaderLayout.setWidthFull();

        getMatchdayLayout().setPadding(false);
        getMatchdayLayout().add(matchdayHeaderLayout, getGrid());
        getMatchdayLayout().setAlignItems(FlexComponent.Alignment.CENTER);
        getMatchdayLayout().setWidth("");

        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column_header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column_header");

        getGrid().addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer1))
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.END)
                .setWidth("13em")
                .setFlexGrow(1);

        getGrid().addColumn((ValueProvider<CalculatedMatch, String>) this::getResultString)
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        getGrid().addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer2))
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("13em")
                .setFlexGrow(1);

        getGrid().setWidth("32em"); // TODO: find a way to set this dynamically based on column widths
        getGrid().setHeightByRows(true);

        getGrid().addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    private String getResultString(CalculatedMatch match) {
        return StringUtils.getResultString(match.getScore1(), match.getScore2());
    }


    /////////////
    // CONTENT //
    /////////////

    @Override
    protected void configureContent() {
        getMatchdayHeader().setText(String.format("Matchday %s", matchdaySelect.getValue().toString()));
        getGrid().setItems(matchService.getCalculatedMatches(matchdaySelect.getValue()));
        configureButtons();
    }

    private void configureButtons() {
        getPrevButton().setEnabled(isMatchDayParamValid(getPrevMatchdayParam()));
        getPrevButton().addClickListener(getButtonClickListener(getPrevMatchdayParam()));

        getNextButton().setEnabled(isMatchDayParamValid(getNextMatchdayParam()));
        getNextButton().addClickListener(getButtonClickListener(getNextMatchdayParam()));
    }

    private ComponentEventListener<ClickEvent<Button>> getButtonClickListener(String matchdayParam) {
        return buttonClickEvent -> getUI().ifPresent(ui -> navigate(ui, seasonParam, matchdayParam));
    }

    private String getPrevMatchdayParam() {
        return String.valueOf(Integer.parseInt(matchdayParam) - 1);
    }

    private String getNextMatchdayParam() {
        return String.valueOf(Integer.parseInt(matchdayParam) + 1);
    }

}
